package com.care.bigscreen.service;

import com.care.bigscreen.vo.*;
import com.care.common.vo.PageResVO;
import com.care.common.util.PageUtil;

import java.util.List;


/**
 * @author stw
 * @version 1.0
 * @description 站点统计表接口实现类
 * @date 2021/05/20/19:41
 */
public interface BigScreenService {

    /**
     * 大屏统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    BigScreenStatisticsVO selectBigScreenStatistics(Long stationId, Long orgId);

    /**
     * 大屏统计查询
     * @return 根据经纬度范围搜索符合条件的大屏统计数据
     */
    BigScreenStatisticsVO selectBigScreenStatisticsByLongitudeLatitude(String longitudeLeftUp,String latitudeLeftUp,String longitudeRightDown,String latitudeRightDown,Long orgId);

    /**
     * 站点列表查询
     */
    List<BigScreenStationVO> selectBigScreenStationList(Long orgId);

    /**
     * 站点列表用户数TOP5查询
     */
    List<BigScreenStationVO> selectBigScreenStationListOlderTop5(Long orgId);

    /**
     * 站点列表安全事件数TOP5查询
     */
    List<BigScreenStationVO> selectBigScreenStationListEventTop5(Long orgId);


    /**
     * 服务站大屏的站点统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    BigScreenStationStatisticsVO selectStationBigScreenStatisticsByStation(Long stationId);

    /**
     * 告警用户列表查询
     */
    List<BigScreenAlarmOlderVO> selectBigScreenAlarmOlderList(Long stationId,String olderName);

    /**
     * 服务站大屏的被监控人详细查询
     */
    BigScreenOlderDetailVO selectStationBigScreenOlderDetail(Long olderId);

    /**
     * 服务站大屏的事件总计查询
     */
    BigScreenEventTotalVO selectStationBigScreenEventTotal(Long stationId);

    /**
     * 服务站大屏的实时安全事件列表查询
     */
    List<BigScreenEventVO> selectBigScreenRtEventList(Long stationId);

    /**
     * 服务站大屏的已完成安全事件列表分页查询
     */
    PageUtil<BigScreenEventVO> selectBigScreenHisEventListByPage(Long stationId, PageResVO page);

    /**
     * 服务站大屏的用户卡片总计查询
     */
    BigScreenOlderCardTotalVO selectStationBigScreenOlderCardTotal(Long stationId,String olderName);

    /**
     * 服务站大屏的用户卡片列表分页查询
     */
    PageUtil<BigScreenOlderDetailVO> selectBigScreenOlderCardListByPage(Long stationId, String olderName,String isAlarm, PageResVO page);

    /**
     * 调用websocket推送实时事件标识给前端
     */
    void pushRtEventFlag(String stationId);
}